USE tempdb;
GO

CREATE LOGIN Boston1 WITH PASSWORD = 'x', CHECK_POLICY = OFF;
CREATE USER Boston1 FROM LOGIN Boston1 WITH DEFAULT_SCHEMA = Sales;
GO

CREATE LOGIN Boston2 WITH PASSWORD = 'x', CHECK_POLICY = OFF;
CREATE USER Boston2 FROM LOGIN Boston2 WITH DEFAULT_SCHEMA = Person;
GO

CREATE TABLE dbo.ErrorLog(id INT);
GO
GRANT SELECT ON dbo.ErrorLog TO Boston1, Boston2;
GO

DBCC FREEPROCCACHE;
GO

/*
-- run this batch once, then add dbo. prefix
*/

EXECUTE AS User = 'Boston1';
GO
-- add dbo. prefix and run again:
SELECT ErrorTime, ErrorMessage FROM ErrorLog;
GO
REVERT;
GO
EXECUTE AS User = 'Boston2';
GO
-- add dbo. prefix and run again:
SELECT ErrorTime, ErrorMessage FROM ErrorLog;
GO
REVERT;
GO



-- now check the plan cache; how many plans?

SELECT t.[text], p.size_in_bytes, p.usecounts, pa.value, 
  [schema] = SCHEMA_NAME(CONVERT(INT, pa.[value]))
FROM sys.dm_exec_cached_plans AS p
CROSS APPLY sys.dm_exec_sql_text(p.plan_handle) AS t
CROSS APPLY sys.dm_exec_plan_attributes(p.plan_handle) AS pa
WHERE t.[text] LIKE N'%Error'+'Log%' AND pa.attribute = N'user_id';

GO
DROP TABLE dbo.ErrorLog;
DROP USER Boston1;
DROP USER Boston2;
DROP LOGIN Boston1;
DROP LOGIN Boston2;
